/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/


#ifndef VARPIPlugin_h
#define VARPIPlugin_h

#ifndef VARPIPlugin_h
#include "VARPIPlugin.h"
#endif VARPIPlugin_h

#ifndef ZzTypes_h
#include "../../../Zz/ZzTypes.h"
#endif //ZzTypes_h

//#ifndef ArmeArchivable_h
//#include "ArmeArchivable.h"
//#endif //ArmeArchivable_h

// number of in/outs is hardcoded for now
#define VARPIMaxInputs 4
#define VARPIMaxOutputs 4

// this is a demo version
//#define VARPIDEMO

//$$$TODO think about locking issues!!!

class VARPIPluginClass;

class VARPIPlugin
{
public :

						VARPIPlugin(const VARPIPluginClass* mother);
 	virtual ~VARPIPlugin() {};

    // returns mother class
	const VARPIPluginClass*	GetClass() const {return fClass;}

	// called by the framework to a plugin parameter needs to be changed.
    bool			IncrParamValue(SInt32 param, float val);


    // setup plugin audio data environment
    // global props
	void			SetBufferNbFrames(SInt32 i) { fBufferNbFrames = i; }
	void 			SetFrameRate(float f) { fFrameRate = f; }

    // input/output props
	void			SetInputChannels(UInt32 i, SInt32 chans) { fInBufferNbChannels[i] = chans; }
	void			SetOutputChannels(UInt32 i, SInt32 chans) { fOutBufferNbChannels[i] = chans; }
	void			SetInputBuffer(UInt32 i, float *buf) { fInBufferPtr[i] = buf; }
	void			SetOutputBuffer(UInt32 i, float *buf) { fOutBufferPtr[i] = buf; }
	
	// set/unset bypass (warning: bypass only works for the ProtectedProcessFrames call)
	void			SetByPass(bool bypass);
	
	

	
protected :
    // retrieve plugin audio data environment
    // global properties
	SInt32			GetBufferNbFrames() const { return fBufferNbFrames; }
	float 			GetFrameRate() const { return fFrameRate; }
    
    // audio input/output properties
	SInt32			GetInputChannels(UInt32 i) const { return fInBufferNbChannels[i]; }
	SInt32			GetOutputChannels(UInt32 i) const { return fOutBufferNbChannels[i]; }
	float*			GetInputBuffer(UInt32 i) const { return fInBufferPtr[i]; }
	float*			GetOutputBuffer(UInt32 i) const { return fOutBufferPtr[i]; }
    


public :
    // methods related to plugin protection
    // this is what the host should call if this is a demo version
    void 	ProtectedProcessFramesAdd(SInt32 nbFrames);
    void 	ProtectedProcessFramesOverwrite(SInt32 nbFrames);
    
private:
    // internal plugin methods
    void		OutputBuffersDemoMute(SInt32 nbFrames);
    void		OutputBuffersClear(SInt32 nbFrames);
    void		OutputBuffersFade(SInt32 nbFrames, float initialGain);
	
	void		CrossfadeInputOutput(SInt32 nbFrames, float init_source_amp, float source_amp_inc);
	
public :
	// Called when the global properties have been changed
    // override to perform reallocation, build lookup tables, etc... 
    // return true if the format suits your plugin.
	virtual bool	FormatChange();
	
    // get/set methods for control parameters
	virtual bool	SetParamValue(SInt32 param, float val) = 0;
	virtual float	GetParamValue(SInt32 param) const = 0;

    // plugins have to implement this function
    // it should process the data in the inputchannels and add to the data
    // in the output channels
    virtual void 	ProcessFramesAdd(SInt32 nbFrames) = 0;
    
    // same as ProcessFramesAdd only the output should be overwritten instead of summed
    // the default implementation clears the outputbuffers and calls ProcessFramesAdd
    // very small processors should implement this method for efficiency (it eliminates a zeroing pass)
    virtual void 	ProcessFramesOverwrite(SInt32 nbFrames);

	// reset plugin state
	virtual void	Reset();


/*

	virtual void			_reserved05();
    virtual void			_reserved06();
	virtual void			_reserved07();
	virtual void			_reserved08();
	virtual void			_reserved09();
	virtual void			_reserved10();
	virtual void			_reserved11();
	virtual void			_reserved12();
	virtual void			_reserved13();
	virtual void			_reserved14();
	virtual void			_reserved15();
	virtual void			_reserved16();
	virtual void			_reserved17();
	virtual void			_reserved18();
	virtual void			_reserved19();
	virtual void			_reserved20();
	virtual void			_reserved21();
	virtual void			_reserved22();
	virtual void			_reserved23();
	virtual void			_reserved24();
	virtual void			_reserved25();
	virtual void			_reserved26();
	virtual void			_reserved27();
	virtual void			_reserved28();
	virtual void			_reserved29();
	virtual void			_reserved30();
	virtual void			_reserved31();
	virtual void			_reserved32();
*/
protected :
    // mother
	const VARPIPluginClass*	fClass;
private :
    // audio environment
    float*					fInBufferPtr[VARPIMaxInputs];
    float*					fOutBufferPtr[VARPIMaxOutputs];
    SInt32					fInBufferNbChannels[VARPIMaxInputs];
    SInt32					fOutBufferNbChannels[VARPIMaxOutputs];
	SInt32					fBufferNbFrames;
	float					fFrameRate;
	SInt32					fClockIncrementFactor;

	// bypass state vars
	bool					fByPass;
	bool					fLastByPass;
    
    // demo clock varables
    UInt32 					fClockState; 		// clock
    bool 					fSoundAmplitudeZero;	// indicates if sound was off
};

#define DECLARE_PLUGIN_ARCHIVE_METHODS(CLASS_NAME) \
virtual bool WriteToArchive(ArmeArchiveHandler* archiver) const; \
CLASS_NAME(ArmeArchiveHandler* unarchiver);

#define IMPLEMENT_PLUGIN_ARCHIVE_METHODS(CLASS_NAME) \
bool CLASS_NAME::WriteToArchive(ArmeArchiveHandler* archiver) const \
{ \
	VARPIPlugin::WriteToArchive(archiver); \
	for(SInt32 i = 0; i < GetClass()->GetNbParams(); i++) \
	{ \
		archiver->WriteFloat(GetParamValue(i)); \
	} \
	return true; \
} \
CLASS_NAME::CLASS_NAME(ArmeArchiveHandler* unarchiver) : \
	VARPIPlugin(unarchiver) \
{ \
	fClass = CLASS_NAME##Class::GetInstance();\
	for(SInt32 i = 0; i < GetClass()->GetNbParams(); i++) \
	{ \
		SetParamValue(i,unarchiver->ReadFloat()); \
	} \
} 
	
#endif
